             .title        "Tengen 800042 Plugin"


;01/02/06
;Written by BootGod
             
;vectors for CopyNES BIOS calls
send_byte:   .equ 0200h		;sends byte to client
baton:       .equ 0203h		;indicates xfer progress on LCD if installed
wr_ppu:      .equ 020ch		;sets address for VRAM access

;variables
read_addr:   .equ 00e0h
read_addr_lo:.equ 00e0h
read_addr_hi:.equ 00e1h
banks_left:  .equ 00e2h
cur_bank:    .equ 00e3h

;plugin header defines
p_ver: = 1	;plugin version
p_read: = 1	;plugin is for downloading (dumping)
p_write: = 2	;plugin is for uploading (RAM carts / WRAM etc)
p_other: = 3	;plugin is for some other purpose (i.e. BankWatch)
p_var1: = 4	;plugin uses user variable #1
p_var2: = 8	;plugin uses user variable #2
p_var3: = 16	;plugin uses user variable #3
p_var4: = 32	;plugin uses user variable #4

;plugin header data

             .org 0380h
             
             ;plugin description
             .db "Tengen 800042 carts "
             .db "(Afterburner)"
             .fill 03e0h-*,00h

             ;plugin creator(s)
             .db "BootGod"
             .fill 03f8h-*,00h

             ;PRG/CHR/WRAM min/max sizes supported
             .db 128 >> 4,0 >> 4
             .db 256 >> 3,0 >> 3
             .db 0,0

             ;plugin version and flags
             .db p_ver
             .db p_read

;plugin code start here
             lda #4		;mapper controlled mirroring
             jsr send_byte

;Set up for PRG transfer

prg_info:    lda #0		;128K PRG
             jsr send_byte
             lda #2
             jsr send_byte
             lda #1		;send type is PRG
             jsr send_byte

             lda #8		;# of 16K banks
             sta banks_left
             lda #0
             sta cur_bank

set_prg_bank:lda cur_bank
             sta 0f000h		;write bank num to PRG select register ($F000)
             inc cur_bank
             lda #0		;set read address to $8000
             sta read_addr_lo
             lda #080h
             sta read_addr_hi

             ldx #64		;set counters for 16K bank xfer
             ldy #0

send_prg:    lda (read_addr),y
             jsr send_byte
             iny
             bne send_prg
             inc read_addr_hi
             jsr baton
             dex
             bne send_prg
             dec banks_left
             bne set_prg_bank	;16K has been sent, load next bank
             
;Set up for CHR transfer
                         
chr_info:    lda #0		;256K CHR
             jsr send_byte
             lda #4
             jsr send_byte
             lda #2		;send type is CHR
             jsr send_byte

             lda #128		;# of 2K banks
             sta banks_left
             lda #0
             sta cur_bank

set_chr_bank:lda cur_bank
             sta 08000h		;write bank num to CHR select ($8000)

             lda #0		;init ppu read
             jsr wr_ppu
             lda 2007h

             ldx #8		;set counters for 2K bank xfer
             ldy #0

send_chr:    lda 2007h
             jsr send_byte
             iny
             bne send_chr
             jsr baton
             dex
             bne send_chr
             inc cur_bank
             dec banks_left
             bne set_chr_bank	;2K has been sent, load next bank

;Signal client no more is coming

done:        lda #0
             jsr send_byte
             lda #0
             jsr send_byte
             lda #0
             jsr send_byte

             rts

             .fill 0800h-*,0ffh	;fill rest to get 1K of data

;UNIF ROM packaging

             .db "P00200"
             .db "C00200"
             .db "C10200"

             .end
